/*****************************************************************************

 stdio.d:

 Standard Stream I/O Functions

 Copyright (c) 2009 Yoshinori SUDA.

 This program is free software; you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation; either version 2 of the License, or 
 (at your option) any later version.

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 merchantability or fitness for a particular purpose. 

 See gpl.txt for more details.

 ----------------------------------------------------------------------------

 Public Functions:

 - void write(char c)
 - void write(FILE* fp, char c)
 - void write(char[] str)
 - void write(FILE* fp, char[] str)
 - void writeLine(char[] str)
 - void writeLine(FILE* fp, char[] str)
 - int read(out char c)
 - int read(FILE* fp, out char c)
 - int readLine(ref char[] buf)
 - int readLine(FILE* fp, ref char[] buf)

 ----------------------------------------------------------------------------

 Class Hierarchy:

 - (No Entry)

*****************************************************************************/

module suda.stdio;

public import std.c.stdio;

class StdioException : Exception
{
	this()
	{
		super("Standard Stream I/O Error");
	}
	
	this(char[] message)
	{
		super(message);
	}
}

void write(char c)
{
	write(stdout, c);
}

void write(FILE* fp, char c)
{
	if (fputc(c, fp) == EOF)
		throw new StdioException;
}

void write(char[] str)
{
	write(stdout, str);
}

void write(FILE* fp, char[] str)
{
	foreach (c; str)
		write(fp, c);
}

void writeLine(char[] str)
{
	writeLine(stdout, str);
}

void writeLine(FILE* fp, char[] str)
{
	write(fp, str);
	write('\n');
}

int read(out char c)
{
	return read(stdin, c);
}

int read(FILE* fp, out char c)
{
	int get = fgetc(fp);
	if (get == EOF) {
		c = 0;
		return 0;
	}
	else {
		c = cast(char)get;
		return 1;
	}
}

int readLine(ref char[] buf)
{
	return readLine(stdin, buf);
}

int readLine(FILE* fp, ref char[] buf)
{
	const int BUF_LEN = 64;
	
	if (buf.length < BUF_LEN)
		buf.length = BUF_LEN;
	int p = 0;
	for (;;) {
		int c = fgetc(fp);
		if (c == EOF || c == '\n')
			break;
		if (p >= buf.length)
			buf.length = buf.length * 2;
		buf[p++] = cast(char)c;
	}
	buf.length = p;
	return p;
}
